@@ -34,7 +34,7 @@ function ManualPulldown(parent, trigger, content) {
             }
             if (!this.activeID) this.activeID = ++MousePopup.nextID;
             MousePopup.activePopups[this.activeID] = this;
-        }
+        };
         var Self = this;
         var recursion = {};
         this.clickHandler = function(e) {
@@ -176,6 +176,52 @@ ComboBox.prototype = {
     }
 };
 
+var phoneFields = ["cellular_telephone1", "cellular_telephone2", "telephone_ip",
+                   "telephone_business1", "telephone_business2",
+                   "telephone_callback", "telephone_car", "telephone_company",
+                   "telephone_home1", "telephone_home2", "telephone_other",
+                   "telephone_isdn", "telephone_pager", "telephone_primary",
+                   "telephone_radio", "telephone_telex", "telephone_ttytdd",
+                   "telephone_assistant"];
+var phoneLabels = [
+    //#. cellular_telephone1
+    _("Mobile"),
+    //#. cellular_telephone2
+    _("Mobile 2"),
+    //#. telephone_ip
+    _("IP phone"),
+    //#. telephone_business1
+    _("Business"),
+    //#. telephone_business2
+    _("Business 2"),
+    //#. telephone_callback
+    _("Callback"),
+    //#. telephone_car
+    _("Car phone"),
+    //#. telephone_company
+    _("Company"),
+    //#. telephone_home1
+    _("Private"),
+    //#. telephone_home2
+    _("Private 2"),
+    //#. telephone_other
+    _("Other"),
+    //#. telephone_isdn
+    _("ISDN"),
+    //#. telephone_pager
+    _("Pager"),
+    //#. telephone_primary
+    _("Primary"),
+    //#. telephone_radio
+    _("Radio"),
+    //#. telephone_telex
+    _("Telex"),
+    //#. telephone_ttytdd
+    _("TTY/TDD"),
+    //#. telephone_assistant
+    _("Assistant")
+];
+
 var dialog = ('<div class="background-color-content" ' +
     'style="width: 25em; padding: 1em; border: 1px solid #555;' +
            'font-size: 10pt; border-radius: 10px; -moz-border-radius: 10px;' +
@@ -288,6 +334,95 @@ var content = dialog.find("textarea");
 var signature = dialog.find("#com\\.openexchange\\.messaging\\.sms\\.signature")
     .click(updateStatus);
 
+// Autocomplete
+var autoCompleteContainer = newnode("div", {
+    position: "absolute", top: "0px", left: "0px", width: "300px", height: "70px", zIndex: 1000,
+    visibility: "hidden", backgroundColor: "white", border: "1px solid #aaa", fontSize: "8pt",
+    overflow: "auto", overflowX: "hidden", overflowY: "auto", MozBoxShadow: "#333 5px 5px 8px"
+});
+document.body.appendChild(autoCompleteContainer);
+
+var autocomplete = new AutoComplete(to.input[0], autoCompleteContainer);
+
+autocomplete.separator = ";";
+
+autocomplete.enableDiv = function() {
+    this.oDiv.style.width = this.oText.offsetWidth + "px";
+    return AutoComplete.prototype.enableDiv.apply(this, arguments);
+};
+
+(function() {
+    var cols = ["500"];
+    for (var i in phoneFields) {
+        cols.push(OXContactMapping.stringmapping[phoneFields[i]]);
+    }
+    autocomplete.columns = cols.join();
+}());
+
+autocomplete.onNewContact = function(data) {
+    for (var i = 1; i < data.length; i++) {
+        if (data[i]) {
+            //#. Autocomplete entry for contacts
+            //#. %1$s is the display name
+            //#. %2$s is the field (type of phone number, e.g. Private or Mobile 2)
+            //#. %3$s is the phone number
+            this.cache.add(format(_("%1$s, %2$s: %3$s"), data[0],
+                phoneLabels[i - 1], data[i]), [data[i]]);
+        }
+    }
+};
+
+autocomplete.additionalRequests = function(pattern) {
+    return [{
+        module: "group",
+        action: "search",
+        data: { pattern: pattern }
+    }];
+};
+
+autocomplete.onAdditionalResponse = function(response, index) {
+    var cache = this.cache;
+    var memberSet = {};
+    for (var i = 0; i < response.data.length; i++) {
+        var group = response.data[i];
+        for (var j = 0; j < group.members.length; j++) {
+            var uid = group.members[j];
+            if (!memberSet[uid]) memberSet[uid] = [];
+            memberSet[uid].push(group.display_name);
+        }
+    }
+    var members = [];
+    for (var i in memberSet) members.push(i);
+    var def = jQuery.Deferred();
+    ox.api.user.getMultiple(members, {
+        columns: "524," + autocomplete.columns,
+        success: function(data) {
+            for (var i in data.data) {
+                var user = data.data[i];
+                var groups = memberSet[user[0]];
+                for (var j = 2; j < user.length; j++) {
+                    if (user[j]) {
+                        cache.add(format(
+                            //#. Autocomplete entry for group members
+                            //#. %1$s is the display name
+                            //#. %2$s is the field (type of phone number, e.g. Private or Mobile 2)
+                            //#. %3$s is the phone number
+                            //#. %4$s is the list of group names which contain the user
+                            ngettext("%1$s, %2$s: %3$s (group %4$s)",
+                                     "%1$s, %2$s: %3$s (groups %4$s)",
+                                     groups.length),
+                            user[1], phoneLabels[j - 2], user[j],
+                            groups.join(", ")), [user[j]]);
+                    }
+                }
+            }
+            def.resolve();
+        },
+        error: function() { def.resolve(); }
+    });
+    return def.promise();
+};
+
 var signatureOption = false, signatureText = null;
 var total = 0;
 var multisms = false;
@@ -298,6 +433,8 @@ var recipientLimit = 1;
 var blacklist = null;
 var numberFilter = /[^0-9#*+]/g;
 
+autocomplete.multiSelect = false;
+
 function updateStatus() {
     if (!total && !smsLimit) return;
     var used = content.val().length;
@@ -355,6 +492,7 @@ ox.JSON.get(AjaxRoot + "/messaging/sms?action=getconfig&session=" + session,
             numberFilter = new RegExp(reply.data.numCleanRegEx, "gi");
         }
         recipientLimit = reply.data.recipientLimit || 1;
+        autocomplete.multiSelect = recipientLimit > 1;
         if (recipientLimit > 1) {
             to.setMultiple(true);
             to.input.attr("title", expectI18n(
@@ -404,52 +542,6 @@ ox.JSON.get(AjaxRoot + "/messaging/sms?action=getconfig&session=" + session,
         }
     });
 
-var phoneFields = ["cellular_telephone1", "cellular_telephone2", "telephone_ip",
-                   "telephone_business1", "telephone_business2",
-                   "telephone_callback", "telephone_car", "telephone_company",
-                   "telephone_home1", "telephone_home2", "telephone_other",
-                   "telephone_isdn", "telephone_pager", "telephone_primary",
-                   "telephone_radio", "telephone_telex", "telephone_ttytdd",
-                   "telephone_assistant"];
-var phoneLabels = [
-    //#. cellular_telephone1
-    _("Mobile"),
-    //#. cellular_telephone2
-    _("Mobile 2"),
-    //#. telephone_ip
-    _("IP phone"),
-    //#. telephone_business1
-    _("Business"),
-    //#. telephone_business2
-    _("Business 2"),
-    //#. telephone_callback
-    _("Callback"),
-    //#. telephone_car
-    _("Car phone"),
-    //#. telephone_company
-    _("Company"),
-    //#. telephone_home1
-    _("Private"),
-    //#. telephone_home2
-    _("Private 2"),
-    //#. telephone_other
-    _("Other"),
-    //#. telephone_isdn
-    _("ISDN"),
-    //#. telephone_pager
-    _("Pager"),
-    //#. telephone_primary
-    _("Primary"),
-    //#. telephone_radio
-    _("Radio"),
-    //#. telephone_telex
-    _("Telex"),
-    //#. telephone_ttytdd
-    _("TTY/TDD"),
-    //#. telephone_assistant
-    _("Assistant")
-];
-
 function open(contact) {
     to.clear();
     signatureText = signatureOption &&
@@ -558,7 +650,7 @@ function uploadMMS() {
                 }, 4.5*60*1000); // 4.5 minutes
             }
         }
-    }
+    };
     mms.form.submit();
 }
 
@@ -636,7 +728,10 @@ jQuery.map(["calendarCreate", "contactsCreate", "infostoreCreate",
             icons: ["plugins/com.openexchange.messaging.sms/images/sms.png"],
             pathPrefix: "",
             big: true,
-            action: function () { open(); }
+            action: function () {
+                open();
+                return true;
+            }
         });
         if (simpleToolbar) section.add(item); else section.insert(item, 1);
     });
